@@ -3,13 +3,14 @@ import fnmatch
 import hashlib
 import logging
 import os
+import platform
 import re
 import shutil
 import tempfile
 from datetime import datetime
 from enum import Enum
 from pathlib import Path
-from typing import List, Dict, Any
+from typing import List, Dict, Any, Tuple, Iterable
 from stat import ST_SIZE
 from stat import ST_MTIME
 
@@ -135,6 +136,7 @@ class FileFinder:
             elif find_type == FindResultType.DIRS:
                 result_files.extend(cls._find_dirs(root, dirs, find_criteria))
             if single_level:
+                # TODO bug: if single level = True, ensure_number_of_results validation won't be executed
                 return result_files
         cls.debug = cls.old_debug
 
@@ -197,11 +199,11 @@ class FileUtils:
         file.close()
 
     @classmethod
-    def prepend_to_file(cls, file, content):
-        with open(file, 'r+') as f:
+    def prepend_to_file(cls, file, line):
+        with open(file, "r+") as f:
             content = f.read()
             f.seek(0, 0)
-            f.write(content)
+            f.write(line.rstrip("\r\n") + "\n" + content)
 
     @classmethod
     def append_data_to_file(cls, path, data):
@@ -547,6 +549,25 @@ class FileUtils:
             result.append(tup)
         return result
 
+    @classmethod
+    def get_creation_time(cls, file_path):
+        """
+        Try to get the date that a file was created, falling back to when it was
+        last modified if that isn't possible.
+        See http://stackoverflow.com/a/39501288/1709587 for explanation.
+        Answer: https://stackoverflow.com/a/39501288/1106893
+        """
+        if platform.system() == "Windows":
+            return os.path.getctime(file_path)
+        else:
+            stat = os.stat(file_path)
+            try:
+                return stat.st_birthtime
+            except AttributeError:
+                # We're probably on Linux. No easy way to get creation dates here,
+                # so we'll settle for when its content was last modified.
+                return stat.st_mtime
+
     @classmethod
     def get_file_sizes_with_mod_dates_in_dir(cls, dir):
         return cls._get_files_with_attrs_in_dir(dir, [ST_SIZE, ST_MTIME])
@@ -839,6 +860,17 @@ class FileUtils:
         tmp = tempfile.NamedTemporaryFile(**kwargs)
         return tmp.name
 
+    @classmethod
+    def write_bytesio_to_file(cls, filename, bytesio):
+        """
+        Write the contents of the given BytesIO to a file.
+        Creates the file or overwrites the file if it does
+        not exist yet.
+        """
+        with open(filename, "wb") as outfile:
+            # Copy the BytesIO stream to the output file
+            outfile.write(bytesio.getbuffer())
+
 
 class JsonFileUtils:
     @classmethod
@@ -852,54 +884,71 @@ class JsonFileUtils:
         if not os.path.isdir(dirname):
             raise ValueError("Should have a dir in path, not a file: {}".format(dirname))
 
-        LOG.info("Starting to write to file: %s", path)
+        bytes_written = -1
+        LOG.trace("Starting to write to file: %s", path)
         with open(path, "w") as file:
             kwargs = {"sort_keys": True}
             if pretty:
                 kwargs["indent"] = 4
             json.dump(data, file, **kwargs)
-        LOG.info("Finished writing to file: %s", path)
+            bytes_written = file.tell()
+        LOG.trace("Finished writing to file: %s", path)
+        return bytes_written
 
     @classmethod
     def load_data_from_json_file(
         cls, file, create_if_not_exists=False, swallow_file_not_found=False, swallow_value_error=False
-    ):
+    ) -> Tuple[Any, int]:
         import json
 
         try:
             with open(file, "r") as f:
-                return json.load(f)
+                data = json.load(f)
+                bytes_read = f.tell()
+                return data, bytes_read
         except FileNotFoundError as e:
             LOG.exception("Error while opening file: %s", file)
             if create_if_not_exists:
                 LOG.info("Creating new empty file: %s", file)
                 FileUtils.create_new_empty_file(file)
             if swallow_file_not_found:
-                return None
+                return None, 0
             raise e
         except ValueError as e:
             LOG.exception("Error while reading file: %s", file)
             if swallow_value_error:
-                return None
+                return None, 0
             raise e
 
 
 class CsvFileUtils:
     @classmethod
-    def append_row_to_csv_file(cls, path, data, header=None):
-        parent_dir = FileUtils.get_parent_dir_name(path)
-        if not FileUtils.is_dir(parent_dir, throw_ex=False):
-            FileUtils.create_new_dir(parent_dir)
+    def append_rows_to_csv_file(cls, file_path, data: List[Iterable[Any]], header=None):
+        import csv
 
+        # Validation
         if not isinstance(data, list):
             raise ValueError("Expected list of data for CSV row!")
+        if len(data) > 0 and not all([isinstance(item, str) for item in data]):
+            raise ValueError("Expected list of str items for CSV row!")
 
-        new_file = True if not os.path.exists(path) else False
-
-        with open(path, "a", newline="") as csvfile:
-            import csv
-
+        new_file = cls._ensure_parent_dir_exists(file_path)
+        with open(file_path, "a", newline="") as csvfile:
             csv_writer = csv.writer(csvfile, delimiter=";", quotechar="|", quoting=csv.QUOTE_MINIMAL)
             if new_file and header:
                 csv_writer.writerow(header)
-            csv_writer.writerow(data)
+            for row in data:
+                csv_writer.writerow(row)
+
+    @classmethod
+    def append_row_to_csv_file(cls, file_path, data: List[str], header=None):
+        data = [data]
+        CsvFileUtils.append_rows_to_csv_file(file_path, data, header=header)
+
+    @classmethod
+    def _ensure_parent_dir_exists(cls, path):
+        parent_dir = FileUtils.get_parent_dir_name(path)
+        if not FileUtils.is_dir(parent_dir, throw_ex=False):
+            FileUtils.create_new_dir(parent_dir)
+        new_file = False if os.path.exists(path) else True
+        return new_file
