@@ -63,6 +63,7 @@ class SimpleLoggingSetupInputConfig:
     project_name: str
     logger_name_prefix: str
     debug: bool = False
+    trace: bool = False
     console_debug: bool = False
     format_str: str = None
     file_postfix: str = None
@@ -71,6 +72,7 @@ class SimpleLoggingSetupInputConfig:
     remove_existing_handlers: bool = True
     disable_propagation: bool = True
     enable_logging_setup_debug_details: bool = False
+    with_trace_level: bool = True
 
     # Dynamic fields
     specified_file_log_level: int or None = None
@@ -88,12 +90,14 @@ class SimpleLoggingSetup:
         logger_name_prefix: str,
         execution_mode: ExecutionMode,
         console_debug=False,
+        trace=False,
         postfix: str = None,
         repos=None,
         verbose_git_log=False,
         format_str=None,
         sanity_check_number_of_handlers=True,
         enable_logging_setup_debug_details: bool = False,
+        with_trace_level: bool = True,
     ) -> SimpleLoggingSetupConfig:
         if not project_name:
             raise ValueError("Project name must be specified!")
@@ -107,12 +111,14 @@ class SimpleLoggingSetup:
             project_name=project_name,
             logger_name_prefix=logger_name_prefix,
             debug=True,
+            trace=trace,
             console_debug=console_debug,
             format_str=final_format_str,
             file_postfix=postfix,
             execution_mode=execution_mode,
             sanity_check_number_of_handlers=sanity_check_number_of_handlers,
             enable_logging_setup_debug_details=enable_logging_setup_debug_details,
+            with_trace_level=with_trace_level,
         )
         SimpleLoggingSetup._setup_gitpython_log(repos, verbose_git_log)
         return logging_config
@@ -134,6 +140,7 @@ class SimpleLoggingSetup:
         project_name: str,
         logger_name_prefix: str,
         debug: bool = False,
+        trace: bool = False,
         console_debug: bool = False,
         format_str: str = None,
         file_postfix: str = None,
@@ -143,11 +150,13 @@ class SimpleLoggingSetup:
         sanity_check_number_of_handlers: bool = True,
         disable_propagation: bool = True,
         enable_logging_setup_debug_details: bool = False,
+        with_trace_level: bool = True,
     ) -> SimpleLoggingSetupConfig:
         conf = SimpleLoggingSetupInputConfig(
             project_name,
             logger_name_prefix,
             debug,
+            trace,
             console_debug,
             format_str,
             file_postfix,
@@ -156,14 +165,20 @@ class SimpleLoggingSetup:
             remove_existing_handlers,
             disable_propagation,
             enable_logging_setup_debug_details,
+            with_trace_level,
         )
+        if trace:
+            conf.with_trace_level = True
+        if conf.with_trace_level:
+            SimpleLoggingSetup.add_logging_level("TRACE", logging.DEBUG - 5, strict=False)
+
         conf.specified_file_log_level = logging.DEBUG if debug else DEFAULT_LOG_LEVEL
+        conf.specified_file_log_level = logging.TRACE if trace else conf.specified_file_log_level
         specified_file_log_level_name: str = logging.getLevelName(conf.specified_file_log_level)
         default_file_log_level_name: str = logging.getLevelName(DEFAULT_LOG_LEVEL)
+        console_log_level = DEFAULT_LOG_LEVEL
         console_log_level: int = logging.DEBUG if debug else DEFAULT_LOG_LEVEL
-
-        if not console_debug:
-            console_log_level = DEFAULT_LOG_LEVEL
+        console_log_level: int = logging.TRACE if trace else console_log_level
 
         final_format_str, formatter = SimpleLoggingSetup._determine_formatter(format_str)
 
@@ -263,6 +278,59 @@ class SimpleLoggingSetup:
             h.setFormatter(formatter)
         return console_handler, handlers, log_file_paths
 
+    @staticmethod
+    def add_logging_level(level_name, level_num, method_name=None, strict=True):
+        """
+        Comprehensively adds a new logging level to the `logging` module and the
+        currently configured logging class.
+
+        `levelName` becomes an attribute of the `logging` module with the value
+        `levelNum`. `methodName` becomes a convenience method for both `logging`
+        itself and the class returned by `logging.getLoggerClass()` (usually just
+        `logging.Logger`). If `methodName` is not specified, `levelName.lower()` is
+        used.
+
+        To avoid accidental clobberings of existing attributes, this method will
+        raise an `AttributeError` if the level name is already an attribute of the
+        `logging` module or if the method name is already present
+
+        Example
+        -------
+        >>> SimpleLoggingSetup.add_logging_level('TRACE', logging.DEBUG - 5)
+        >>> logging.getLogger(__name__).setLevel("TRACE")
+        >>> logging.getLogger(__name__).trace('that worked')
+        >>> logging.trace('so did this')
+        >>> logging.TRACE
+        5
+
+        Source: https://stackoverflow.com/a/35804945/1106893
+        """
+        if not method_name:
+            method_name = level_name.lower()
+
+        if strict:
+            if hasattr(logging, level_name):
+                raise AttributeError("{} already defined in logging module".format(level_name))
+            if hasattr(logging, method_name):
+                raise AttributeError("{} already defined in logging module".format(method_name))
+            if hasattr(logging.getLoggerClass(), method_name):
+                raise AttributeError("{} already defined in logger class".format(method_name))
+
+        # This method was inspired by the answers to Stack Overflow post
+        # http://stackoverflow.com/q/2183233/2988730, especially
+        # http://stackoverflow.com/a/13638084/2988730
+        def logForLevel(self, message, *args, **kwargs):
+            if self.isEnabledFor(level_num):
+                self._log(level_num, message, args, **kwargs)
+
+        def logToRoot(message, *args, **kwargs):
+            logging.log(level_num, message, *args, **kwargs)
+
+        logging.addLevelName(level_num, level_name)
+        setattr(logging, level_name, level_num)
+        setattr(logging.getLoggerClass(), method_name, logForLevel)
+        setattr(logging, method_name, logToRoot)
+
     @staticmethod
     def _determine_formatter(format_str):
         final_format_str = DEFAULT_FORMAT
